# ==============================================================================
# Information Analysis
#
# This is the set of data analyses used in Drozdova and Gaubatz 
# Quantifying the Qualitative: Information Theory for 
# Comparative Case Analysis. (SAGE 2015)
#
#  In each case, the data is in a tab-delimited file containing the data
#  matrix with the variable names in the first row and the case names in 
#  the first column. The Dependent (outcome) variable should be in the 
#  last column.
#
#  All values must be binary {0,1}
#
#
#                         Kurt Taylor Gaubatz, Katya Drozdova - November 3, 2015
#
# ==============================================================================

# ==============================================================================
# ===== Options ================================================================
# ==============================================================================

options(scipen= 6)                      # Turn off scientific notation

# ==============================================================================
# ===== Functions ==============================================================
# ==============================================================================

# ===== Entropy Function to perform information analysis =======================
# This version uses conditional and joint probabilities for the calculations.
# Input is a dataframe with cases in the rows and variables in the columns.
# The first column is the case names. The first row is the variable names.
# The last column should be the dependent variable. All values must be {0,1}.
#
# To facilitate some of the calculations, I set up an inverted copy 
#   switching 0/1 in the dataframe edat2.

entropy = function(edata){

  edat1 = edata                        # Copy dataframe w/o case names
  edat2 = abs(edat1-1)                 # Create copy w/ inverted zeros and ones
  out1 = edat1[dim(edat1)[2]]          # Set up Y variable for outcomes
  out2 = edat2[dim(edat2)[2]]          # Set up Y variable w/inverted outcomes
  edat1 = edat1[-dim(edat1)[2]]        # Set up X variables alone
  edat2 = edat2[-dim(edat2)[2]]        # Set up inverted X variables alone
  n = dim(edat1)[1]                    # Get number of cases

  # ===== Joint Probabilities ==================================================
  # The joint probabilities are the number of times the two values occur divided
  # by the total number of cases

  jp00 = sapply(edat2*out2[,1], sum)/n # Joint pr x=0 & y=0
  jp11 = sapply(edat1*out1[,1], sum)/n # Joint pr x=1 & y=1
  jp01 = sapply(edat2*out1[,1], sum)/n # Joint pr x=0 & y=1
  jp10 = sapply(edat1*out2[,1], sum)/n # Joint pr x=1 & y=0

  # ===== Total Probabilities for each Attribute ===============================
  # The total probability that x=0 is the sum of the joint probabilities of x=0
  # when y=0 and x=0 when y=1 etc.

  tp0 = jp00 + jp01                    # Total probability x=0
  tp1 = jp11 + jp10                    # Total probability x=1

  # ===== Conditional Probabilities ============================================
  # The conditional probability is the probability that y is a specific value
  # given that x is a specific value.

  cp00 = jp00/tp0                      # Conditional pr y=0 given x=0
  cp01 = jp10/tp1                      # Conditional pr y=0 given x=1
  cp10 = jp01/tp0                      # Conditional pr y=1 given x=0
  cp11 = jp11/tp1                      # Conditional pr y=1 given x=1

  # ===== NA Substitutions =====================================================
  # Where probabilities or logs are undefined, replace with 0 since as x 
  # approaches zero, xlogx approaches 0.  

  cp00 = ifelse(is.na(cp00), 0, cp00)  # Change NA to 0
  cp01 = ifelse(is.na(cp01), 0, cp01)
  cp10 = ifelse(is.na(cp10), 0, cp10)
  cp11 = ifelse(is.na(cp11), 0, cp11)

  # Jiggle 0's to .00001 to keep logs defined

  cp00 = ifelse(cp00==0, .00001, cp00)
  cp01 = ifelse(cp01==0, .00001, cp01)
  cp10 = ifelse(cp10==0, .00001, cp10)
  cp11 = ifelse(cp11==0, .00001, cp11)

  # ===== Conditional Information Entropy ======================================
  H = -tp0 * (cp00*log2(cp00)  +       # Conditional Entropy for outcome Y
              cp10*log2(cp10)) -       #   given X.
       tp1 * (cp01*log2(cp01)  +          
              cp11*log2(cp11))

  # ===== Mutual Information (uncertainty reduction in outcome Y due to X) =====

  pY0 = sum(out2)/n                    # Total probability Y = 0
  pY1 = sum(out1)/n                    # Total probability Y = 1
  HY  = -pY0*log2(pY0) - pY1*log2(pY1) # Outcome uncertainty for Y

  MI = HY - H                          # Mutual Information
  
  # ===== Create a variable to record direction of relationship (pos or neg)
  
  D = ifelse(jp00 + jp11 > jp01 + jp10, 1, -1)
  
  Hout = cbind(HY, H, MI, D)           # Output Total Entropy, Cond Entropy 
  return(Hout)                         #  Mutual Information, and direction
}                                      # End of Entropy Function

# ==============================================================================
# ==============================================================================
# This is another version of the entropy function that uses the abcd method
# and computes the entropy scores for just any two variables. Both variables 
# have to be binary {0,1} and have the same number of observations.

entropy2 = function(x, y){

  n = length(x)                        # Get number of cases

  # ===== Joint Probabilities ==================================================
  # The joint probabilities are the number of times the two values occur divided
  # by the total number of cases. We will set this up as the following matrix:
  # 
  #          X  
  #         1 0
  #  Y = 1  a b
  #      0  c d
  #
  a = sum(x*y)/n                       # Joint pr x=1 & y=1
  b = sum((!x)*y)/n                    # Joint pr x=0 & y=1
  c = sum(x*(!y))/n                    # Joint pr x=1 & y=0
  d = sum((!x)*(!y))/n                 # Joint pr x=0 & y=0
   
  # ===== NA Substitutions =====================================================
  # Where probabilities or logs are undefined, replace with 0 since as x 
  # approaches zero, xlogx approaches 0.  

  a = ifelse(is.na(a), 0, a)           # Change NA to 0
  b = ifelse(is.na(b), 0, b)
  c = ifelse(is.na(c), 0, c)
  d = ifelse(is.na(d), 0, d)

  # Jiggle 0's to .00001 to keep logs defined

  a = ifelse(a==0, .00001, a)
  b = ifelse(b==0, .00001, b)
  c = ifelse(c==0, .00001, c)
  d = ifelse(d==0, .00001, d)

  # ===== Conditional Information Entropy ======================================
  HYgX = -(d*(log2(d) - log2(b+d)) + b*(log2(b) - log2(b+d))) -
          (c*(log2(c) - log2(a+c)) + a*(log2(a) - log2(a+c)))
  # This is a streamlined version of the following: 
  #  HYgX = -(b+d)*((d/(b+d))*log2(d/(b+d)) + (b/(b+d))*log2(b/(b+d))) -
  #          (a+c)*((c/(a+c))*log2(c/(a+c)) + (a/(a+c))*log2(a/(a+c)))
    
  # Total Information Entropy for Y ============================================
  HY = -(c+d)*log2(c+d) -              # Outcome uncertainty for Y
        (a+b)*log2(a+b)
  
  # ===== Mutual Information (uncertainty reduction in outcome Y due to X) =====
  MI = HY - HYgX                       # Mutual Information
  
  # ===== Create a variable to record direction of relationship (pos or neg)
  
  D = ifelse(a+d > b+c, 1, -1)
  
  Hout = cbind(HY, HYgX, MI, D)        # Package Cond Entropy, Mutual Info & D 
  return(Hout)                         # Output Cond Entropy, Mutual Info & D
}                                      # End of Entropy2 Function

# ==============================================================================
# ===== Print Function =========================================================
# This function prints out the information metrics from the entropy function

printH=function(results){              # A function for printing results
  print.output = cat("Information Analysis Results","\n",
    "Datafile: ", filename, "\n",      # Show data file name
    "Dependent variable: ",            # Show dependent variable
    colnames(rawdata[dim(rawdata)[2]]), "\n\n",
    "HY   = H(Y) = Information entropy for Y","\n",
    "HYgX = H(Y|X) = Conditional Information Entropy","\n",
    "MI   =  Mutual Information = Uncertainty Reduction in Y due to X","\n",
    "D    = Direction: Positive or Negative relationship between Y and X", 
    "\n\n")
    results  
}


# ==============================================================================
# ===== dotchart2 Function =====================================================
# This is a custom version of the R dotchart plotting function. We use this for 
# The Chapter 6 dotcharts. It facilitates the bold group titles and lets the 
# user separate the cex (size) option for the text labels and the cex option 
# for the plotting symbols (The normal R dotchart function combines these 
# parameters). 
# The new parameters are cex.font= for the text labels, cex.pch= for the symbol
# sizes, and cex.gpch= for the group symbol sizes.
# This function also changes the y axis labels to include a 1.0 at the end of 
# the series.
# We've added a few comments for guidance, but most of this is just straight
# from the basic R function, which you can see in the R console by just 
# entering "dotchart" without any parentheses.

dotchart2 = function(x, labels = NULL, 
    groups = NULL, gdata = NULL, 
    cex = par("cex"), 
    pch = 21, gpch = 21, 
    bg = par("bg"), color = par("fg"), 
    gcolor = par("fg"), 
    lcolor = "gray", 
    cex.font=1, cex.pch=1, cex.gpch=1, # The new parameters
    xlim = range(x[is.finite(x)]), 
     main = NULL, 
    xlab = NULL, ylab = NULL, ...) 
{
    opar <- par("mai", "mar", "cex", "yaxs")
    on.exit(par(opar))                 # Reset the par options on exit
    par(cex = cex, yaxs = "i")

    if (!is.numeric(x))                # Check that input is a numeric vector
        stop("'x' must be a numeric vector or matrix")
    n <- length(x)
    if (is.matrix(x)) {
        if (is.null(labels))           # Setup Y axis labels
            labels <- rownames(x)
        if (is.null(labels)) 
            labels <- as.character(1L:nrow(x))
        labels <- rep_len(labels, n)
        if (is.null(groups))           # Set up group axis labels
            groups <- col(x, as.factor = TRUE)
        glabels <- levels(groups)
    }
    else {
        if (is.null(labels)) 
            labels <- names(x)
        glabels <- if (!is.null(groups)) 
            levels(groups)
        if (!is.vector(x)) {
            warning("'x' is neither a vector nor a matrix: using as.numeric(x)")
            x <- as.numeric(x)
        }
    }
    plot.new()                         # Start the Plot
    linch <- if (!is.null(labels))     # Set up Y axis labels
        max(strwidth(labels, "inch")*cex.font, na.rm = TRUE)
    else 0
    if (is.null(glabels)) {
        ginch <- 0
        goffset <- 0
    }
    else {
        ginch <- max(strwidth(glabels, "inch"), na.rm = TRUE)
        goffset <- 0.4                
    }
    if (!(is.null(labels) && is.null(glabels))) {
        nmai <- par("mai")
        nmai[2L] <- nmai[4L] + max(linch + goffset, ginch) + 
            0.1
        par(mai = nmai)
    }
    if (is.null(groups)) {
        o <- 1L:n
        y <- o
        ylim <- c(0, n + 1)
    }
    else {
        o <- sort.list(as.numeric(groups), decreasing = TRUE)
        x <- x[o]
        groups <- groups[o]
        color <- rep_len(color, length(groups))[o]
        lcolor <- rep_len(lcolor, length(groups))[o]
        offset <- cumsum(c(0, diff(as.numeric(groups)) != 0))
        y <- 1L:n + 2 * offset
        ylim <- range(0, y + 2)
    }
    plot.window(xlim = xlim, ylim = ylim, log = "")
    lheight <- par("csi")
    if (!is.null(labels)) {
        linch <- max(strwidth(labels, "inch")*cex.font, na.rm = TRUE)
        loffset <- (linch + 0.1)/lheight
        labs <- labels[o]
        mtext(labs, side = 2, line = loffset, at = y, adj = 0, 
            col = color, las = 2, cex = cex*cex.font, ...)
    }
    abline(h = y, lty = "dotted", col = lcolor)
    points(x, y, pch = pch, col = color, bg = bg, 
            cex=cex*cex.pch,           # Adjust size of dots
            xpd=NA)                    # Allow to print over margin
    if (!is.null(groups)) {
        gpos <- rev(cumsum(rev(tapply(groups, groups, length)) + 
            2) - 1)
        ginch <- max(strwidth(glabels, "inch"), na.rm = TRUE)
        goffset <- (max(linch + 0.2, ginch, na.rm = TRUE) + 0.5)/lheight
        mtext(glabels, side = 2, line = goffset, at = gpos, adj = 0, 
            col = gcolor, las = 2, font=2, cex = cex, ...)
        if (!is.null(gdata)) {
            abline(h = gpos, lty = "dotted")
            points(gdata, gpos, pch = gpch, col = gcolor, bg = bg, 
            cex=cex*cex.gpch,          # Adjust size of group dots
                xpd=TRUE, ...)         # Allow over margin for dots
        }
    }
    axis(side= 1, at= c(0,.2,.4,.6,.8,1), labels= c(0,.2,.4,.6,.8,1.0), xpd= NA)
    box()
    title(main= main, xlab= xlab, ylab= ylab, ...)
    invisible()
}

# ==============================================================================
# ===== Some Generic Routines ==================================================
# ==============================================================================

# These are generic routines that can be used with cut-and-paste to set up 
# the analysis of different datasets. The only thing you need to change is
# the filename in the first line. Change "myData.txt" to match the name of 
# your dataset.

# ===== Setup Data =============================================================
#
# This is a generic routine for setting up and reviewing the data.
#
filename = "myData.txt"                # Filename with data
rawdata = read.delim(filename,         # Setup a dataframe with the data
  header= TRUE,                        # Data has header
  stringsAsFactors= FALSE)             # Don't let strings convert to factors   

names(rawdata)                         # Show variable names
row.names(rawdata) = rawdata[,1]       # Set row names (cases) from 1st column
rawdata = rawdata[,-1]                 # Drop first column of case names
vars = names(rawdata)                  # List of all variable names
ivList = vars[-length(vars)]           # List of independent variable names
cases = row.names(rawdata)             # Create list of cases from row names
ncases = dim(rawdata)[1]               # Get number of cases
nvars = dim(rawdata)[2]                # Get number of variables
cat("Case List: \n",                   # Show cases
  cases, sep= "\n\t")
cat("Variable List: \n",               # Show variables
    vars, sep= "\n\t")


# ===== Display Results ========================================================
# This is a generic routine for using the printH function to display the 
# results of the entropy function
results = data.frame(entropy(rawdata)) # Run Entropy Function
results = round(                       # Round results to 3 decimal places
  results[order(results$MI,            #   and sort from largest to smallest
  decreasing= T),], 3)

printH(results)  

# ==============================================================================
# ===== Chapter Two ============================================================
# ==============================================================================

# ----- Chapter Two Figure 1 - Entropy Curve -----------------------------------

p = seq(0,1,.0001)                     # x values
H = -p*log2(p) -(1-p)*log2(1-p)        # y values (H)

png(filename= "fig2-2.png",            # Output a png file
    units= "in",
    res= 1200,
    width= 6,
    height= 4)

par(mai= c(1,1,.25,.25))               # Set margins in inches (B,L,T,R)
 
# Create an empty plot
plot(x= 0, y= 0, type= "n",            # Plot nothingness
  xlab= "p", ylab= "H",                # No axis labels
  yaxs= "i",                           # No overage on the Y axis
  yaxt= "n", xaxt= "n",                # Turn off x and y axes
  xlim= c(0, 1),                       # Set range of x axis
  ylim= c(0, 1.1))                     # Set range of y axis

axis(side= 1,                          # Set up X axis
  at= seq(0, 1, .1),   
  cex.axis= .75)

axis(side= 2,                          # Set up y axis
  at= seq(0, 1, .1),
  cex.axis= .75,
  las= 1)
     
# Create a medium gray rectange over the plot area
rect(                                  # Use plot area coordinates
    xleft= par()$usr[1],               # Left side x coordinate
    ybottom= par()$usr[3],             # Bottom y coordinate
    xright= par()$usr[2],              # Right side x coordinate
    ytop= par()$usr[4],                # Top y coordinate
    col= gray(.9))                     # Set the color to a medium gray
 
# Overlay grid lines
grid(                                  # Place grid lines at tick marks
     lty= 1,                           # Use solid lines
     col= "white")                     # Set color to white

abline(v= .5,
       lty= 2,
       col= gray(.5))

# Overlay the plot
points(p, H,                           # plot p on x axis H on y
  type= "l",                           # Set line type
  col= "black")                        # Set color of line

dev.off()                              # finalize file

# ==============================================================================
# ===== Chapter 4 ==============================================================
# ==============================================================================

filename = "George-data.txt"           # Filename with data
rawdata = read.delim(filename,         # Setup a dataframe with the data
  header= TRUE,                        # Data has header
  stringsAsFactors= FALSE)             # Don't let strings convert to factors   

names(rawdata)                         # Show variable names
row.names(rawdata) = rawdata[,1]       # Set row names (cases) from 1st column
rawdata = rawdata[,-1]                 # Drop first column of case names
vars = names(rawdata)                  # List of all variable names
ivList = vars[-length(vars)]           # List of independent variable names
cases = row.names(rawdata)             # Create list of cases from row names
ncases = dim(rawdata)[1]               # Get number of cases
nvars = dim(rawdata)[2]                # Get number of variables
cat("Case List:", cases, sep= "\n   ") # Show cases
cat("Var. List:", vars, sep= "\n   ")  # Show variables

results = data.frame(entropy(rawdata)) # Run Entropy Function
results = round(                       # Round results to 3 decimal places
  results[order(results$MI,            #   and sort from largest to smallest
  decreasing= T),], 3)

printH(results)  

# ==============================================================================
# ===== Chapter 5 ==============================================================
# ==============================================================================

# ----- Example 1- Rudel Ecuador Forests Data ----------------------------------
filename = "Rudel-Ecuador-Forests.txt" # Filename with data
rawdata = read.delim(filename,         # Setup a dataframe with the data
  header= TRUE,                        # Data has header
  stringsAsFactors= FALSE)             # Don't let strings convert to factors   

names(rawdata)                         # Show variable names
row.names(rawdata) = rawdata[,1]       # Set row names (cases) from 1st column
rawdata = rawdata[,-1]                 # Drop first column of case names
vars = names(rawdata)                  # List of all variable names
ivList = vars[-length(vars)]           # List of independent variable names
cases = row.names(rawdata)             # Create list of cases from row names
ncases = dim(rawdata)[1]               # Get number of cases
nvars = dim(rawdata)[2]                # Get number of variables
cat("Case List: \n", cases)            # Show cases
cat("Variable List: \n", vars)         # Show variables

results = data.frame(entropy(rawdata)) # Run Entropy Function
results = round(                       # Round results to 3 decimal places
  results[order(results$MI,            #   and sort from largest to smallest
  decreasing= T),], 3)

printH(results)  


# ----- Example 2 - Teacher Networks Data --------------------------------------
filename = "Coburn-Teacher-Networks.txt" # Filename with data
rawdata = read.delim(filename,         # Setup a dataframe with the data
  header= TRUE,                        # Data has header
  stringsAsFactors= FALSE)             # Don't let strings convert to factors   

names(rawdata)                         # Show variable names
row.names(rawdata) = rawdata[,1]       # Set row names (cases) from 1st column
rawdata = rawdata[,-1]                 # Drop first column of case names
vars = names(rawdata)                  # List of all variable names
ivList = vars[-length(vars)]           # List of independent variable names
cases = row.names(rawdata)             # Create list of cases from row names
ncases = dim(rawdata)[1]               # Get number of cases
nvars = dim(rawdata)[2]                # Get number of variables
cat("Case List: \n", cases)            # Show cases
cat("Variable List: \n", vars)         # Show variables

results = data.frame(entropy(rawdata)) # Run Entropy Function
results = round(                       # Round results to 3 decimal places
  results[order(results$MI,            #   and sort from largest to smallest
  decreasing= T),], 3)

printH(results) 


# ----- Example 3 - Nursing ----------------------------------------------------
filename = "Nursing-Data.txt"          # Filename with data
rawdata = read.delim(filename,         # Setup a dataframe with the data
  header= TRUE,                        # Data has header
  stringsAsFactors= FALSE)             # Don't let strings convert to factors   

names(rawdata)                         # Show variable names
row.names(rawdata) = rawdata[,1]       # Set row names (cases) from 1st column
rawdata = rawdata[,-1]                 # Drop first column of case names
vars = names(rawdata)                  # List of all variable names
ivList = vars[-length(vars)]           # List of independent variable names
cases = row.names(rawdata)             # Create list of cases from row names
ncases = dim(rawdata)[1]               # Get number of cases
nvars = dim(rawdata)[2]                # Get number of variables
cat("Case List: \n", cases)            # Show cases
cat("Variable List: \n", vars)         # Show variables

results = data.frame(entropy(rawdata)) # Run Entropy Function
results = round(                       # Round results to 3 decimal places
  results[order(results$MI,            #   and sort from largest to smallest
  decreasing= T),], 3)

printH(results) 


# ==============================================================================
# ===== Chapter 6: Sensitivity Analysis ========================================
# ==============================================================================

# ----- Set up Waste Data ------------------------------------------------------
filename = "RedWaste.txt"              # Filename with data
rawdata = read.delim(filename,         # Setup a dataframe with the data
         stringsAsFactors= FALSE)      # Don't let strings convert to factors   

names(rawdata)                         # Show variable names
attributes(rawdata)$row.names =        # Set row names (cases)
  rawdata[,1]                          #   
rawdata = rawdata[,-1]                 # Drop community names
vars = names(rawdata)                  # List of variable names
ivList = vars[-length(vars)]           # List of independent variable names
cases = attributes(rawdata)$row.names  # Create list of cases from row names
ncases = dim(rawdata)[1]               # Get number of cases
nvars = dim(rawdata)[2]-1              # Get number of variables
cat("Case List: \n", cases)            # Show cases
cat("Variable List: \n", vars)         # Show variables

# ----- First cut entropy analysis ---------------------------------------------

results = data.frame(entropy(rawdata)) # Run Entropy Function
results = round(                       # Round results to 3 decimal places
  results[order(results$MI,            #   and sort from largest to smallest
  decreasing= T),], 3)

printH(results)  


# ----- Dropped Case Impact Assessment -----------------------------------------
# In this analysis we create a loop running the entropy function over the set
# of cases dropping one case each time. The CaseOut dataframe is a matrix
# where each row holds the Mutual Information score for each variable when 
# a specific case is dropped.

CaseOut = entropy(rawdata)[,3]         # Set up vector with all cases results
for(i in 1:ncases){                    # Run through data dropping each case
  Houtput = entropy(rawdata[-i,])[,3]  # Get results with each case missing
  CaseOut = rbind(CaseOut, Houtput)    # Build output for each run
}                                      # End for() loop
attributes(CaseOut)$dimnames[[1]] =    # Add case names to rows
  c("AllCases", attributes(rawdata)$row.names)

# ----- Dropped Case Variances -------------------------------------------------

# Variance by variable and case

v.diffs = array(NA,                    # Set up an empty array adding a row for
  dim= c(ncases+1, nvars))             #  all cases
for(i in 0:ncases+1){                  # Loop through each case
  v.diffs[i,] =                        # Calculate difference between MI w/all 
    abs(CaseOut[i,] - CaseOut[1,])     #  cases and MI w/case dropped
}

# ----- Table 6.3: Summary of mean & max change by variable for dropping cases

v.means = apply(v.diffs[-1,],          # Mean of abs(differences for each IV)
              2, mean)                 #  dropping the all cases row

v.max = apply(v.diffs[-1,],            # Max of abs(differences for each IV)
              2, max)

cbind(ivList, round(v.means, 2),       # Show mean and max change for each var 
      round(v.max, 2))                 #  from dropping cases one at a time

# ----- Table 6.4: Summary of mean & max change by case for dropping cases

c.means = apply(v.diffs[,-nvars],      # Mean of abs(differences for each case)
  1, mean) 
c.max = apply(v.diffs[,-nvars],        # Max of abs(differences for each case)
  1, max)  

cbind(cases, round(c.means[-1], 2),    # Show mean change for all variables for 
      round(c.max[-1], 2))             # each dropped case (skip 1st row, which
                                       # is the value without any dropped case.



# ----- Dropped Case Graphics --------------------------------------------------

# ----- Figure 6.1: A dropped case dotchart ------------------------------------

png(filename= "fig6-1.png",
    units= "in",
    res= 1200,
    width= 6,
    height= 4)

par(                                   # Set graphics parameters
  mfrow= c(1, 1),                      # Set up 1 row for each IV 
  omi= c(.5, .5, .5, .5),              # Set outer margins on page
  mar= c(0, 0, 0, 0),                  # Set zero margins for all plots
  xaxt= "s",                           # Turn off x axes
  xaxs= "i"                            # Turn off overage on x axis
)

dotchart2(CaseOut[-1, c(3, 4, 6, 8)],  # Use CaseOut minus line for all data
  gdata= CaseOut[1, c(3, 4, 6, 8)],    # Grouping data--composite data
  xlim= c(0, 1),
  cex= .8,
  cex.font= .65,                       # Set font size
  cex.pch= 1,
  cex.gpch= 1.25,
  pch= 1,
  gpch= 19,                            # Set grouping data symbol
  gcolor= "black"                      # Color of grouping data
)

dev.off()

# ----- Fig 6.2: Another dropped case dotchart emphasizing Glendale ------------

png(filename= "fig6-2.png",
    units= "in",
    res= 1200,
    width= 6,
    height= 4)

par(                                   # Set graphics parameters
  mfrow= c(1, 1),                      # Set up 1 row for each IV 
  omi= c(.5, .5, .5, .5),              # Set outer margins on page
  mar= c(0, 0, 0, 0),                  # Set zero margins for all plots
  xaxt= "s",                           # Turn off x axes
  xaxs= "i"                            # Turn off overage on x axis
)

dotchart2(CaseOut[-1, c(3, 10, 11)],   # Use CaseOut minus line for all data
  gdata= CaseOut[1, c(3, 10, 11)],     # Grouping data--composite data
  xlim= c(0, 1),
  cex= .8,
  cex.font= .65,                       # Set font size
  cex.pch= 1,
  cex.gpch= 1.25,
  pch= 1,
  gpch= 19,                            # Set grouping data symbol
  gcolor= "black"                      # Color of grouping data
)

oldpar = par()
par(usr= c(0, 1, 0, 1))

abline(h= c(.06, .3943, .727),         # Add gray highlighting line 
  lwd= 8, col= "#00000030")            #   for Glendale cases

par(oldpar)

dev.off()

# ----- Some additional analytic charts, not included in the book --------------

# A chart of dropped case effects with all of the variables
dotchart2(CaseOut[-1,],                # Use CaseOut minus line for all data
  gdata= CaseOut[1,],                  # Grouping data--composite data
  xlim= c(0, 1),
  cex= .6,                             # Set font size
  pch= 1,
  gpch= 19,                            # Set grouping data symbol
  gcolor= "red"                        # Color of grouping data
)

# ----- A chart to show mean differences by variable when cases are dropped ----
par(mfrow= c(2, 1))
dotchart(v.means,
  pch= 19,
  xlim= c(0,1),
  labels= vars,
  main= "Dropped Cases: \nMean differences by Variable")

# ----- A chart to show mean differences by case when cases are dropped --------
dotchart(c.means[-1],
  pch= 19,
  xlim= c(0, 1),
  labels= cases,
  main= "Dropped Cases: \nMean differences by Case")


# ==============================================================================
# ===== Chapter 7 - The QCA Connection ========================================= 
# ==============================================================================

# ----- QCA Analysis -----------------------------------------------------------
#
# This uses the R QCA package to find prime implicants in dataset
#
# Note that it is much easier if variable names are single capital letters
# This is not setup as fully generic code. It requires customization to 
# account for the specific prime implicants derived in the QCA step.
#
library(QCA)                           # Load the QCA package

# ----- Analysis of Ragin's Ethnic Political Mobilization Data -----------------

filename = "Ragin-ethnic.txt"          # Identify filename with data
QCA.Data =                             # Read in data
  read.delim(filename, 
  header= T, stringsAsFactors= FALSE)
row.names(QCA.Data) = QCA.Data[,1]     # Set row names to first column values
QCA.Data = QCA.Data[,-1]               # Drop case names column
nvars = dim(QCA.Data)[2]               # Set number of variables


# Information analysis ---------------------------------------------------------

results =                              # Run Entropy Function
  data.frame(entropy(QCA.Data))
results = round(                       # Round results to 3 decimal places
  results[order(results$MI,            #   and sort from largest to smallest
  decreasing= T),], 3)

printH(results)                        # Show the information results

# QCA Analysis -----------------------------------------------------------------

eqmcc(QCA.Data, "E1", details= TRUE, show.cases= TRUE)

# QCA + Information ------------------------------------------------------------
# Code for prime implicant sets and collect in a data frame (QCA.Data2).

LW = ifelse(QCA.Data$L == 1 & QCA.Data$W == 1, 1, 0)
SG = ifelse(QCA.Data$S == 1 & QCA.Data$G == 1, 1, 0)
SW = ifelse(QCA.Data$S == 1 & QCA.Data$W == 0, 1, 0)
lwG = ifelse(QCA.Data$L == 0 & QCA.Data$W == 0 & QCA.Data$G==1, 1, 0)

ANY = ifelse(LW | SG | lwG, 1, 0)

QCA.Data2 = data.frame(LW, SG, SW, lwG, ANY, QCA.Data$E1)

results = data.frame(entropy(QCA.Data2)) # Run Entropy Function
results = round(                       # Round results to 3 decimal places
  results[order(results$MI,            #   and sort from largest to smallest
  decreasing= T),], 3)

printH(results)                        # Show info. analysis on QCA results


# Analysis of Ragin/Rokkan's data on the role of the Russian revolution --------
# Read data --------------------------------------------------------------------
filename = "rokkan-ragin-Data.csv"
QCA.Data = read.csv(filename, header= T, stringsAsFactors= FALSE)
row.names(QCA.Data) = QCA.Data[,1]     # Set row names to first column values
QCA.Data = QCA.Data[,-1]               # Drop case names column
nvars = dim(QCA.Data)[2]

# Information analysis ---------------------------------------------------------

results=data.frame(entropy(QCA.Data))  # Run Entropy Function
results = round(                       # Round results to 3 decimal places
  results[order(results$MI,            #   and sort from largest to smallest
  decreasing= T),], 3)

printH(results)  

# QCA Analysis -----------------------------------------------------------------

eqmcc(QCA.Data, "S", details= TRUE, show.cases= TRUE)

# QCA + Information ------------------------------------------------------------
# Code for prime implicant sets and collect in data frame (QCA.Data2).

rle = ifelse(QCA.Data$R==0 & QCA.Data$L==0 & QCA.Data$E==0, 1, 0)
crE = ifelse(QCA.Data$C==0 & QCA.Data$R==0 & QCA.Data$E==1, 1, 0)
CRLe = ifelse(QCA.Data$C==1 & QCA.Data$R==1 & 
                  QCA.Data$L==1 & QCA.Data$E==1, 1, 0)

ANY = ifelse(rle | crE | CRLe, 1, 0)

QCA.Data2 = data.frame(rle, crE, CRLe, ANY, QCA.Data$S)

results = data.frame(entropy(QCA.Data2)) # Run Entropy Function
results = round(                       # Round results to 3 decimal places
  results[order(results$MI,            #   and sort from largest to smallest
  decreasing= T),], 3)

printH(results) 

# Assess marginal information contributions for fle recipe 1. rl together:

rl = ifelse(QCA.Data$R==0 & QCA.Data$L==0, 1, 0)
entropy2(rl,QCA.Data$S)
re = ifelse(QCA.Data$R==0 & QCA.Data$E==0, 1, 0)
entropy2(re,QCA.Data$S)
le = ifelse(QCA.Data$E==0 & QCA.Data$L==0, 1, 0)
entropy2(le,QCA.Data$S)











 